@@ -4,281 +4,22 @@ Created on Sat Mar  5 20:57:19 2022
 
 @author: Christian
 
-Problems - make sure that hatching is consistent at different beam scales.
-Make sure that units support imperial
-
-Hatching should be similar to beam size?
-
 """
 
 import matplotlib.pyplot as plt
 import numpy as np
+from dataclasses import dataclass
 from matplotlib.patches import Rectangle, Polygon, Circle, FancyArrowPatch
-
 from abc import ABC, abstractmethod
- 
-# hatches = ['/', '\\', '|', '-', '+', 'x', 'o', 'O', '.', '*']
+from planesections import diagramUnits
 
 
-
-# fig, axs = plt.subplots(2, 5, constrained_layout=True, figsize=(6.4, 3.2))
-
-# =============================================================================
-# Archetype classes
-# =============================================================================
-
-
-"""
-These archetype classes represent what a 3rd part tool will need to have
-to plug into the functions.
-They should define all the meaningful interfaces/abstractions in the class.
-
-"""
-
 """
 Not used properly right now, but ideally this is defined only in one place.
 """
-# fixities  = {'free':0, 'roller': 1, 'pinned':2, 'fixed':3}
 fixities  = {'free':[0,0,0], 'roller': [0,1,0], 'pinned':[1,1,0], 'fixed':[1,1,1]}
 
 
-
-
-class NodeArchetype:
-    """
-    Represents a node/point of interst on the diagram.
-    """
-    @abstractmethod
-    def getFixityType():
-        """
-        Gets the fixity type used.
-        Currently supported - free, roller, pin, fixed.
-        """
-        pass
-    
-    @abstractmethod
-    def getPosition():
-        """
-        Gets the x position of the node.
-        Currently supported - roller, pin, fixed.
-        """
-
-        pass
-    
-    @abstractmethod
-    def getLabel():
-        """
-        Returns the label that has been assigned to the node/
-        ???:
-            what values are supported??
-        """
-        pass    
-    
-
-
-class BeamArchetype:
-    """
-    Archetype class of data beams need in order to use make use of the beam
-    diagram class
-    
-    ???:
-        Should nodes be given a required internal naming convention?
-    """
-    nodes = []
-    NodeArchetype = None
-    forces = []
-    distForces = []
-
-    
-    
-    @abstractmethod
-    def getNodes():
-        """
-        Returns all noed objects in the beam.
-        """
-        pass
-    
-    @abstractmethod
-    def getForces():
-        """
-        Gets all applied point Loads.
-        """        
-        pass
-    
-    @abstractmethod
-    def getDistForses():
-        """
-        Gets all applied distrubuted Loads.
-        """ 
-        pass    
-    
-    
-    @abstractmethod
-    def setNodeArchetype():
-        """
-        Sets the archetype method used to create new nodes.
-        """ 
-        pass
-    
-    @abstractmethod
-    def addNode(self, xCoord, fixity, label=None):
-        """
-        Adds a new node to the model.
-        """ 
-        pass    
-
-        
-        
-        
-
-
-# ???
-# call point force?
-
-class ForceArchetype:
-    """
-    Represents a force applied to a single node
-    """
-    
-    @abstractmethod
-    def getConnectedNode():
-        """
-        """
-        pass
-    
-    @abstractmethod
-    def getMagnitudes():
-        """
-        Returns the magnitude of force at each of the connected nodes
-        """
-        pass   
-    
-     
-    @abstractmethod
-    def getLabelNodes():
-        """
-        Returns the nodes that are labeled
-        """
-        pass         
-
-class DistrubtedForceArchetype:
-    """
-    Represents a force distributed across multiple nodes, i.e. uniform, etc
-    """
-    
-    @abstractmethod
-    def getConnectedNode():
-        """
-        
-        """
-        pass
-    
-    @abstractmethod
-    def getMagnitudes():
-        """
-        Returns the magnitude of force at each of the connected nodes
-        """
-        pass   
-    
-     
-    @abstractmethod
-    def getLabelNodes():
-        """
-        Returns the nodes that are labeled
-        """
-        pass      
-    
-    
-
-
-# =============================================================================
-# Implementations.
-# =============================================================================
-
-
-class Node(NodeArchetype):
-    """
-    A concrete implementation for a class that represents a node/point of 
-    interst on the diagram.
-    """
-    
-    def __init__(self, xCoord, fixity, label):
-        self.x = xCoord
-        self.fixity = fixity
-        self.label = label
-    
-    
-    def getFixityType(self):
-        """
-        Gets the fixity type used.
-        Currently supported - free, roller, pin, fixed.
-        """
-        return self.fixity
-    
-    def getPosition(self):
-        """
-        Gets the x position of the node.
-        Currently supported - roller, pin, fixed.
-        """
-
-        return self.x
-    
-    def getLabel(self):
-        """
-        Returns the label that has been assigned to the node/
-        ???:
-            what values are supported??
-        """
-        return self.label  
-
-
-class Beam(BeamArchetype):
-    
-    def __init__(self):
-        """
-        The concrete implementaiton of the beam class.
-
-        """
-        nodes = []
-        NodeArchetype = None
-        forces = []
-        distForces = []
-        
-        self.setNodeArchetype()
-    
-    def getNodes(self):
-        """
-        Returns all noed objects in the beam.
-        """
-        return self.nodes
-    
-    def getForces(self):
-        """
-        Gets all applied point Loads.
-        """        
-        return self.forces
-    
-    def getDistForses(self):
-        """
-        Gets all applied distrubuted Loads.
-        """ 
-        print('NOT SUPPORTED YET')            
-        
-        
-    def addNode(self, xCoord, fixity, label=None):
-        NewNode = self.nodeArchetype(xCoord, fixity,label)
-        self.nodes.append(NewNode)
-    
-    def setNodeArchetype(self, nodeArchetype:NodeArchetype = Node):
-        """
-        Used to set the type of node class used. The node should inherit from
-        the nodeArhetype class.
-        """
-        self.nodeArchetype = nodeArchetype                
-    
-    
-    
-
 # =============================================================================
 # 
 # =============================================================================
@@ -315,15 +56,13 @@ class BasicBeamDiagram(BeamDiagram):
     def __init__(self, scale = 1, supScale = 0.8):
 
         self.lw = 1 * scale
-        
         self.scale = scale # Scales all drawing elements
-        # self.yScale = 1
         
         # changes the offset from the point in x/y
         self.labelOffset = 0.1*scale
         
         # Pin geometry variables
-        self.r = 0.1*scale*supScale
+        self.r = 0.08*scale*supScale
         self.hTriSup = 0.3*scale*supScale
         self.wTriSup = 2*self.hTriSup
         
@@ -339,9 +78,9 @@ class BasicBeamDiagram(BeamDiagram):
         self.y0 = 0
 
         self.supportPlotOptions = {'fixed':self.plotFixed, 
-                                 'pinned':self.plotPinned,
-                                 'roller':self.plotRoller,
-                                 'free':self.plotFree}
+                                   'pinned':self.plotPinned,
+                                   'roller':self.plotRoller,
+                                   'free':self.plotFree}
 
     def _initPlot(self, figSize, xlims, ylims, dpi = 300):
         
@@ -481,7 +220,6 @@ class BasicBeamDiagram(BeamDiagram):
 
         """
         xy0Rect, xyLine = self._getFixedSupportCords(xy0, isLeft)
-        # print(isLeft)
         self.ax.add_patch(Rectangle(xy0Rect, self.hFixedRect, self.wRect, ec='black', fc='white', hatch=self.hatch, lw = self.lw))
         self.ax.plot(xyLine[0], xyLine[1], c = 'white', lw = self.lw)
 
@@ -582,7 +320,11 @@ class BasicBeamDiagram(BeamDiagram):
         y = xy0[1] + self.labelOffset
         self.ax.text(x, y, label, {'size':12*self.scale})
         # self.labelOffset
-
+        
+    # def plotForceLabel(self, xy0, label):
+    #     x = xy0[0] + self.labelOffset
+    #     y = xy0[1] + self.labelOffset
+    #     self.ax.text(x, y, label, {'size':12*self.scale})
 
 
 # =============================================================================
@@ -592,16 +334,18 @@ class BasicBeamDiagram(BeamDiagram):
 
 
 class BeamPlotter2D:
-    def __init__(self, beam, figsize = 8):
+    def __init__(self, beam, figsize = 8, units = 'environment'):
         """
         Used to make a diagram of the beam. Only certain fixities are supported
         for plotting, including free, roller (support only in y), pinned (support in x and y),
         and fixed (support in x/y/rotation).
         Only certain forces are supported for plotting - for distrubuted
         forces only the y component of the beam can be plotted.
+        Mixed Forces, those that contain a combination of Px/Py/Moment will 
+        not be plotted
      
-        Note, the diagram has been rescaled so the beam has lenght scaled to the
-        maximum beam size of 8.
+        Note, the diagram has been rescaled so it's length in the digram 
+        isn't it's analysis lenght.
         This is to make consistent plotting easier across a number of beam sizes,
         however, the matplotlib objects in the plot have different value than 
         the actual beam.
@@ -609,12 +353,18 @@ class BeamPlotter2D:
         The class used as a interface between the high level beam abstraction 
         and lower level rules plotting.    
         
-        
+        unitHandler: str, or dict
+            Represents the . the env tag.
+            
         """        
         
         self.beam = beam
         self.figsize = figsize
-        
+        if units == 'environment':
+            self.unitHandler = diagramUnits.activeEnv
+        else:
+            self.unitHandler = diagramUnits.getEnvironment(units)
+            
         L = beam.getLength()       
         xscale = beam.getLength()  / self.figsize
         self.xscale = xscale
@@ -624,9 +374,11 @@ class BeamPlotter2D:
         self.xmin = xlims[0]
         self.xmax = xlims[0]
                 
-        xlimsPlot = [(xlims[0] - L/20) / xscale, (xlims[1] + L/20) / xscale]
-        ylimsPlot = [-L/10 / xscale, L/10 / xscale]
-        self.plotter._initPlot(self.figsize, xlimsPlot, ylimsPlot)
+        self.xlimsPlot = [(xlims[0] - L/20) / xscale, (xlims[1] + L/20) / xscale]
+        self.ylimsPlot = [-L/10 / xscale, L/10 / xscale]
+        
+        self.plottedNodeIDs = []
+        
         
     def plotBeam(self):
         xlims   = self.beam.getxLims()
@@ -639,11 +391,9 @@ class BeamPlotter2D:
         for node in self.beam.nodes:
             fixityType  = node.getFixityType()
             x = node.getPosition()
-            
             """
             This makes some big assumptions about the shape of the system.
             """
-            
             kwargs = {}
             if fixityType == 'fixed' and x == self.xmin:
                 kwargs =  {'isLeft':True}
@@ -656,7 +406,8 @@ class BeamPlotter2D:
             # plot the appropriate option without an if statement!
             self.plotter.supportPlotOptions[fixityType](xy, **kwargs)
 
-    def plot(self, **kwargs):
+    def plot(self, plotLabel = False, labelForce = True, 
+             plotForceValue = True, **kwargs):
         """
         Plots the beam, supports, point forces, element forces, and labels.
         
@@ -670,13 +421,22 @@ class BeamPlotter2D:
 
         """
         
-        self.plotBeam()
+        self.plotter._initPlot(self.figsize, self.xlimsPlot, self.ylimsPlot)
         self.plotSupports()
         if self.beam.pointLoads:
-            self.plotPointForces()
+            fplot = self.plotPointForces()
+        if self.beam.pointLoads and plotLabel:
+            self.plotPointForceLables(fplot, labelForce, plotForceValue)
+            
         if self.beam.eleLoads:
-            self.plotEleForces()
-        self.plotLabels()
+            fplot, xcoords = self.plotEleForces()
+        if self.beam.eleLoads and plotLabel:
+            self.plotDistForceLables(fplot, xcoords, labelForce, plotForceValue)
+            
+        if plotLabel:
+            # print('here')
+            self.plotLabels()
+        self.plotBeam()
         
     def plotLabels(self):
         """
@@ -685,10 +445,105 @@ class BeamPlotter2D:
         """
         for node in self.beam.nodes:
             label = node.label
-            if label:
-                x = node.getPosition()
+            x     = node.getPosition()
+            
+            if label and (self._checkIfLabelPlotted(node.ID) != True):
                 xy = [x / self.xscale, 0]
                 self.plotter.plotLabel(xy, label)
+                self._addLabelToPlotted(node.ID)
+
+    def _getValueText(self, diagramType, forceValue):
+        
+        unit = self.unitHandler[diagramType].unit
+        scale = self.unitHandler[diagramType].scale
+        Ndecimal = self.unitHandler[diagramType].Ndecimal
+        
+        # Round force
+        forceValue *= scale
+        if Ndecimal == 0:
+            forceValue = round(forceValue)
+        else:
+            forceValue = round(forceValue*10**Ndecimal) / 10**Ndecimal
+        return forceValue, unit
+
+    def _checkIfLabelPlotted(self, nodeID):
+        check = nodeID in self.plottedNodeIDs
+        return check
+        
+    def _addLabelToPlotted(self, nodeID):
+        self.plottedNodeIDs.append(nodeID)
+
+    def plotPointForceLables(self, fplot, labelForce, plotForceValue):
+        """
+        Adds all labels to the plot. Labels are offset from the point in the 
+        x and y.
+        """
+        
+        inds = range(len(self.beam.pointLoads))
+        for ii, force in zip(inds, self.beam.pointLoads):
+            Px, Py, Mx = fplot[ii]
+            isMoment = False
+            if Mx != 0:
+                isMoment = True
+                Py = -0.15
+                diagramType = 'moment'
+                fText = force.P[2]
+            else:
+                # shift the force down so it fits in the diagram!
+                Py += 0.15
+                diagramType = 'force'
+                fText = np.sum(force.P[:2]**2)**0.5
+            
+            # get the label from the node - it's store there and not on the force.
+            labelBase = self.beam.nodes[force.nodeID - 1].label
+            label = ''
+            
+            if labelBase and labelForce and isMoment:
+                label +=  f'$M_{{{labelBase}}}$' # Tripple brackets needed to make the whole thing subscript
+                
+            elif labelBase and labelForce and (not isMoment):
+                label += f'$P_{{{labelBase}}}$'
+            else:
+                pass
+            
+            if labelBase and plotForceValue and labelForce:
+                valueText, unit = self._getValueText(diagramType, fText)
+                label += ' = ' + str(valueText) + "" + unit
+                        
+            x = force.getPosition()
+            xy = [x / self.xscale, -Py]
+            
+            if label and self._checkIfLabelPlotted(force.nodeID) != True:
+                self.plotter.plotLabel(xy, label)
+                self._addLabelToPlotted(force.nodeID)
+            
+                              
+                            
+    def plotDistForceLables(self, fplot, xcoords, labelForce, plotForceValue):
+        """
+        Adds all labels to the plot. Labels are offset from the point in the 
+        x and y.
+        """
+        diagramType = 'distForce'
+        inds = range(len(self.beam.eleLoads))
+        for ii, force in zip(inds, self.beam.eleLoads):
+            qx, qy = fplot[ii]
+            fText = force.P[1]
+            
+            labelBase = force.label
+            label = ''
+            
+            if labelBase and labelForce:
+                label += f'$q_{{{labelBase}}}$'
+            
+            if labelBase and plotForceValue and labelForce:
+                valueText, unit = self._getValueText(diagramType, fText)
+                label += ' = ' + str(valueText) + "" + unit            
+            
+            x1, x2 = xcoords[ii]
+            aMid = (x1 + x2 ) / 2
+            xy = [aMid, -qy]   # note, aMid has already been scaled         
+            self.plotter.plotLabel(xy, label)              
 
     def _getForceVectorLength(self, forces, vectScale = 1):
         """
@@ -730,21 +585,18 @@ class BeamPlotter2D:
         This prevents small from being plotted that look silly.
 
         """
-        forces = []
+        forces  = []
         xcoords = []
-        
         for force in self.beam.pointLoads:
             forces.append(force.P)
             xcoords.append(force.x / self.xscale)
-        fplot = self._getForceVectorLength(forces)
+        fplot  = self._getForceVectorLength(forces)
         NLoads = len(forces)
+        
         for ii in range(NLoads):
             Px, Py, Mx = fplot[ii]
-            
             x = xcoords[ii]
-            
-            # if it's a moment, plot it as a moment
-            if (Px == 0 and Py ==0):
+            if (Px == 0 and Py ==0): # if it's a moment, plot it as a moment
                 if Mx < 0:
                     postive = True
                 else:
@@ -752,6 +604,8 @@ class BeamPlotter2D:
                 self.plotter.plotPointMoment(x, postive)
             else:
                 self.plotter.plotPointForce(x - Px, -Py, Px, Py)
+                
+        return fplot
 
     def plotEleForces(self):
         """
@@ -762,8 +616,7 @@ class BeamPlotter2D:
         
         # The spacing between force lines
         spacing = self.beam.getLength() / 25 / self.xscale
-        
-        forces = []
+        forces  = []
         xcoords = []        
         for force in self.beam.eleLoads:
             forces.append(force.P)
@@ -784,8 +637,10 @@ class BeamPlotter2D:
                 print("WARNING: Distributed load has no vertical component.")            
             else:
                 # This is a little akward, but Py is added to account for the offset of -Py in the base funciton.
-                self.plotter.plotVerticalLineLoad(x1, x2, Py, y1 + Py, spacing=spacing)           
-           
+                self.plotter.plotVerticalLineLoad(x1, x2, Py, y1 + Py, spacing=spacing)
+        return fplot, xcoords
+   
+    
     def _getStackedPositions(self, xcoords: list[list[float]], fplot):
         """
         Gives the forces an order, and finds where to put them porportionally.
@@ -797,10 +652,9 @@ class BeamPlotter2D:
         xcoords = np.array(xcoords)
         ycoords = np.zeros(Nforces) # [bottom, top]
         
-        # Get the lengths - we want to 
+        # Get the lengths
         lengths = xcoords[:,1] - xcoords[:,0]
         sortedInds = np.argsort(lengths)[::-1]
-        
         fplotOut = np.zeros_like(fplot)
         fplotOut[:] = fplot
 
@@ -815,10 +669,8 @@ class BeamPlotter2D:
             y0 = self._getStackedDatum(xcoords[ind], plottedxCoords, plottedyCoords)
 
             ycoords[ind] =  y0 - dy
-            
             plottedxCoords.append(xcoords[ind])           
             plottedyCoords.append(ycoords[ind])
-
         return ycoords
     
     def _checkIfInRange(self, xtest, x1,x2):
@@ -843,9 +695,9 @@ class BeamPlotter2D:
         return 0
                        
 
-def plotBeamDiagram(beam):
+def plotBeamDiagram(beam, plotLabel = True, labelForce = False, plotForceValue = False, units = 'environment'):
     """
-    Creates a diagram of the created beam. 
+    Creates a diagram of the created beam.
     Only certain fixities are supported for plotting, including free, roller 
     (support only in y), pinned (support in x and y), and fixed 
     (support in x/y/rotation).
@@ -857,13 +709,34 @@ def plotBeamDiagram(beam):
     This is to make consistent plotting easier across a number of beam sizes,
     however, the matplotlib objects in the plot have different value than 
     the actual beam.
-        
+    
     The resulting diagram is a matplotlib figure that can be further manipulated.
 
     Parameters
     ----------
     beam : PlaneSections beam
         The Beam Object to be plotted.
+    plotLabel : bool, optional
+        A toggle that turns on or off plotting user defined labels along the 
+        beam. The default is True.
+    labelForce : bool, optional
+        A toggle that turns on or off if forces get labeled. If toggled to 
+        true, forces will be given a label instead of the beam location.
+        The default is True.
+    plotForceValue : bool, optional
+        A toggle that turns on or off plotting the force values with the label. 
+        If set to true, the magnitude of the force will be plotted in
+        the digram units. The default is False.
+    units : str, optional
+        A string that specified how the diagram units will be managed in the.
+        The default value 'environment' causes the plot to read from the active
+        environment. see :py:class:`planesections.DiagramUnitEnvironmentHandler`
+        
+        If not set to environment, it can have value:  *'metric',  
+        'metric_kNm',  'metric_Nm',  
+        'imperial_ftkip',  'imperial_ftlb'*        
+
+        The unit handler for the plot. This The default is 'environment'.
 
     Returns
     -------
@@ -872,7 +745,9 @@ def plotBeamDiagram(beam):
     ax : matplotlib ax
 
     """
-    diagram = BeamPlotter2D(beam)
-    diagram.plot()
+    diagram = BeamPlotter2D(beam, units = units)
+    diagram.plot(plotLabel, labelForce, plotForceValue)
     return diagram.plotter.fig, diagram.plotter.ax
 
+
+
